/************************************************************************
 *   IRC - Internet Relay Chat, src/masking.c
 *   Copyright (C) 1998	Dean F. Bailey
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include "h.h"

int tagcount = 0;
#define MAXVIRTSIZE     (3 + 5 + 1)
char *hideme(acptr, sptr)
    aClient *acptr, *sptr;
{

    if (!IsRegistered(acptr)) {
	return "<Unknown>";
    }
    if (IsHidden(acptr)) {
	if (IsNetAdmin(acptr))
	    return netadmin_host;
	else if (IsSRoot(acptr))
	    return sroot_host;
	if (IsAdmin(acptr))
	    return admin_host;
	else if (IsSAdmin(acptr))
	    return sadmin_host;
	else if (IsOper(acptr))
	    return oper_host;
	else if (IsLocOp(acptr))
	    return locop_host;
	else if (!acptr->tag)
	    acptr->tag = tagcount++;
	if (*acptr->user->host && (!MyConnect(acptr) || IsUnixSocket(acptr)))
	    return maskme(acptr->user->host,
			  acptr->tag);
	else
	    return maskme(acptr->sockhost,
			  acptr->tag);
    } else {
	if (*acptr->user->host && (!MyConnect(acptr) || IsUnixSocket(acptr)))
	    return acptr->user->host;
	else
	    return acptr->sockhost;
    }
}
char *maskmenow(acptr, sptr)
    aClient *acptr, *sptr;
{
    if (!acptr->tag)
	acptr->tag = tagcount++;
    if (*acptr->user->host && (!MyConnect(acptr) || IsUnixSocket(acptr)))
	return maskme(acptr->user->host,
		      acptr->tag);
    else
	return maskme(acptr->sockhost,
		      acptr->tag);
}

#define B_BASE                  0x1000

int Maskchecksum(char *data, int len)
{
    int i;
    int j;

    j = 0;
    for (i = 0; i < len; i++) {
	j += *data++;
    }

    return (j + B_BASE) % 0xffff;
}
int str2array(char **pparv, char *string, char *delim)
{
    char *tok;
    int pparc = 0;

    tok = (char *)strtok(string, delim);
    while (tok != NULL) {
	pparv[pparc++] = tok;
	tok = (char *)strtok(NULL, delim);
    }

    return pparc;
}


char *maskme(char *s, int duh)
{
    static char mask[HOSTLEN + 1];
    int csum;
    char *dot, *ptr;
    static char destroy[HOSTLEN + 1], *parv[HOSTLEN + 1];
    int len = 0, overflow = 0, parc = 0;

    strncpyzt(destroy, s, HOSTLEN);
    len = strlen(destroy);

    if ((len + MAXVIRTSIZE) > HOSTLEN) {
	overflow = (len + MAXVIRTSIZE) - HOSTLEN;
	ptr = &destroy[overflow];
    } else {
	ptr = &destroy[0];
    }
    memset(mask, 0, HOSTLEN);
    csum = Maskchecksum(s, strlen(s));

    parc = str2array(parv, ptr, ".");

    if (strlen(s) > HOSTLEN) {
	s[HOSTLEN] = 0;
    }
    if (parc == 4) {
	len = strlen(parv[3]);
	if (isdigit(parv[3][len - 1])) {
	    sprintf(mask, "%s.%s.%s.000", parv[0], parv[1], parv[2]);
	    return mask;
	}
    }
    dot = (char *) strchr(s, '.');

    if (dot == NULL) {
	sprintf(mask, userspace_x_prefix "%d.%s", csum, s);
	return mask;
    } else {
	sprintf(mask, userspace_x_prefix "%d.%s", csum, dot + 1);
	return mask;
    }
}
